
{$APPTYPE GUI}
{$MODE DELPHI}

uses
  Windows, Cairo, CairoWin32;

{$R wincairo.res}

const
  CLASSNAME = 'WinHello';

function WindowProc(hWindow: HWND; aMessage: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall; export;
var
  dc: HDC;
  ps: PAINTSTRUCT;
  r: RECT;
  cr: pcairo_t;
  surface: pcairo_surface_t;
begin
  result := 0;

  case aMessage of
    WM_PAINT:
      begin
        dc := BeginPaint(hWindow, @ps);
        GetClientRect(hWindow, @r);
        
        surface := cairo_win32_surface_create_with_dib(CAIRO_FORMAT_ARGB32, r.right - r.left, r.bottom - r.top);
        
        cr := cairo_create(surface);
        cairo_scale(cr, r.right - r.left, r.bottom - r.top);
        cairo_set_source_rgb(cr, 1, 1, 1);
        cairo_paint(cr);
        cairo_set_source_rgb(cr, 1, 0, 0);
        cairo_set_line_width(cr, 0.1);
        cairo_move_to(cr, 0.2, 0.2);
        cairo_line_to(cr, 0.8, 0.8);
        cairo_stroke(cr);
        cairo_destroy(cr);
        
        BitBlt(
          dc,
          0,
          0,
          r.right - r.left,
          r.bottom - r.top,
          cairo_win32_surface_get_dc(surface),
          0,
          0,
          SRCCOPY
        );
        
        cairo_surface_destroy(surface);
        
        EndPaint(hWindow, ps);
        exit;
      end;
    WM_DESTROY:
      begin
        PostQuitMessage(0);
        exit;
      end;
  end;

  result := DefWindowProc(hWindow, aMessage, wParam, lParam);
end;

function WinRegister: Boolean;
var
  wclass: WNDCLASS;
begin
  wclass.Style := CS_HREDRAW or CS_VREDRAW;
  wclass.lpfnWndProc := WNDPROC(@WindowProc);
  wclass.cbClsExtra := 0;
  wclass.cbWndExtra := 0;
  wclass.hInstance := system.MainInstance;
  wclass.hIcon := LoadImage(hInstance, MAKEINTRESOURCE(1), IMAGE_ICON, 0, 0, 0);
  wclass.hCursor := LoadCursor(0, IDC_ARROW);
  wclass.hbrBackground := GetStockObject(WHITE_BRUSH);
  wclass.lpszMenuName := nil;
  wclass.lpszClassName := CLASSNAME;

  result := RegisterClass(wclass) <> 0;
end;

function WinCreate: HWND;
begin
  result := CreateWindow(
    CLASSNAME,
    pchar(Utf8ToAnsi('Utilisation de la bibliothèque Cairo')),
    WS_OVERLAPPEDWINDOW,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    400 + 2 * GetSystemMetrics(SM_CXFRAME),
    400 + 2 * GetSystemMetrics(SM_CYFRAME) + GetSystemMetrics(SM_CYCAPTION),
    0,
    0,
    system.MainInstance,
    nil
  );

  if result <> 0 then
  begin
    ShowWindow(result, CmdShow);
    ShowWindow(result, SW_SHOW);
    UpdateWindow(result);
  end;
end;

var
  m: MSG;
  h: HWND;

begin
  if not WinRegister then
  begin
    MessageBox(0, 'Register failed', nil, mb_Ok);
    exit;
  end;
  
  h := WinCreate;
  if longint(h) = 0 then
  begin
    MessageBox(0, 'WinCreate failed', nil, mb_Ok);
    exit;
  end;

  while GetMessage(@m, 0, 0, 0) do
  begin
    TranslateMessage(m);
    DispatchMessage(m);
  end;
  
  Halt(m.wParam);
end.
